"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var registry_data_1 = require("./registry-data");
var decorators_1 = require("./decorators");
var Container = (function () {
    function Container(parent) {
        this.parent = parent;
        this.registry = new Map();
    }
    Container.prototype.register = function (provider) {
        var _this = this;
        if (provider instanceof Array) {
            provider.forEach(function (p) { return _this.registerOne(p); });
        }
        else {
            this.registerOne(provider);
        }
    };
    Container.prototype.resolve = function (token) {
        var _this = this;
        var registryData = this.registry.get(token);
        if (!registryData) {
            if (this.parent) {
                return this.parent.resolve(token);
            }
            else {
                throw new Error("No provider for " + token);
            }
        }
        if (registryData.instance) {
            return registryData.instance;
        }
        var cls = registryData.cls;
        var injectionsMd = this.getInjections(cls);
        var resolvedInjections = injectionsMd.map(function (injectionMd) { return _this.resolve(injectionMd.token); });
        var args = [];
        injectionsMd.forEach(function (injection, index) {
            args[injection.parameterIndex] = resolvedInjections[index];
        });
        registryData.instance = new (cls.bind.apply(cls, [void 0].concat(args)))();
        this.registry.set(token, registryData);
        return registryData.instance;
    };
    Container.prototype.createScope = function () {
        return new Container(this);
    };
    Container.prototype.registerOne = function (provider) {
        var token;
        var cls;
        if (typeof provider === 'function') {
            token = provider;
            cls = provider;
        }
        else {
            token = provider.token;
            cls = provider.useClass;
        }
        var registryData = new registry_data_1.RegistryData(cls);
        this.registry.set(token, registryData);
    };
    Container.prototype.getInjections = function (cls) {
        return Reflect.getOwnMetadata(decorators_1.INJECTIONS_MD_KEY, cls) || [];
    };
    return Container;
}());
exports.Container = Container;
